# TEXTRACT -- Extract TRES image into one or two echelle files
# By Doug Mink, SAO Telescope Data Center
# September 18, 2007

procedure textract (images)

	file	images=""	{prompt="Input image(s) to extract"}

	file	aprefsci=""	{prompt="Science aperture reference spectrum"}
	file	aprefsky=""	{prompt="Sky aperture reference spectrum"}
	file	aprefcal=""	{prompt="Calibration aperture reference spectrum"}
	file	apdir=""	{prompt="Directory for reference spectra"}
	string	readnoise="1"	{prompt="Read out noise sigma (photons)"}
	string	gain="1"	{prompt="Photon gain (photons/data number)"}
	real    npeaks=0.2	{prompt="Select brightest peaks"}
	bool	deltemp=yes	{prompt="Delete temporary file (y or n)"}
	bool	verbose=yes	{prompt="Log progress (y or n)"}
	bool	debug=yes	{prompt="Print debugging information (y or n)"}

	begin
	bool vb, db
	file imin, imsky, imsci, imcal, imroot, templist, listfile
	file apsci, apsky, apcal, adir, cwd
	file  apskypath, apskynew, apscipath, apscinew, apcalpath, apcalnew
	file imskypath, imscipath, imcalpath, outsci, outsky, outcal
	int lroot
	string history, ims, tchar, args

	ims = images
	adir = apdir
	fixdir (adir)
	adir = fixdir.newdir
	vb = verbose
	db = debug

# Copy existing aperture maps to local database/ directory
	if (!access ("database/"))
	    mkdir ("database")
	pathname | scan (cwd)

# Get file names for sky reference image and aperture file
	if (aprefsky != "") {
	    smatch (aprefsky, ".fits")
	    if (smatch.found) {
		lroot = smatch.start - 1
		apsky = substr (aprefsky, 1, lroot)
		apsky = "ap" // apsky
		imsky = aprefsky
		}
	    else {
		imsky = aprefsky // ".fits"
		apsky = "ap" // aprefsky
		}

# Copy sky reference aperture file to local database/
	    apskypath = adir // "database/" // apsky
	    apskynew = "database/" // apsky
	    if (!access (apskynew)) {
		print ("TEXTRACT: ",apskypath," => ", apskynew)
		copy (apskypath, apskynew)
		}
	    else
		print ("TEXTRACT: ",apskynew," already exists")

# Copy sky reference image file to local directory
	    imskypath = adir // imsky
	    if (!access (imsky)) {
		print ("TEXTRACT: Copying ",imskypath, " to ",cwd)
		args = osfn (imskypath) // " " // imsky
		lns (args)
		}
	    else
		print ("TEXTRACT: ",imsky," already exists")
	    }

# Get file names for science reference image and aperture file
	if (aprefsci != "") {
	    smatch (aprefsci, ".fits")
	    if (smatch.found) {
		lroot = smatch.start - 1
		apsci = substr (aprefsci, 1, lroot)
		apsci = "ap" // apsci
		imsci = aprefsci
		}
	    else {
		imsci = aprefsci // ".fits"
		apsci = "ap" // aprefsci
		}

# Copy science reference aperture file to local database/
	    apscipath = adir // "database/" // apsci
	    apscinew = "database/" // apsci
	    if (!access (apscinew)) {
		copy (apscipath, apscinew)
		print ("TEXTRACT: ",apscipath," => ", apscinew)
		}
	    else
		print ("TEXTRACT: ",apscinew," already exists")

# Copy science reference image file to local directory
	    imscipath = adir // imsci
	    if (!access (imsci)) {
		print ("TEXTRACT: Copying ",imscipath, " to ",cwd)
		args = osfn (imscipath) // " " // imsci
		lns (args)
		}
	    else
		print ("TEXTRACT: ",imsci," already exists")
	    }

# Get file names for calibration reference image and aperture file
	if (aprefcal != "") {
	    smatch (aprefcal, ".fits")
	    if (smatch.found) {
		lroot = smatch.start - 1
		apcal = substr (aprefcal, 1, lroot)
		apcal = "ap" // apcal
		imcal = aprefcal
		}
	    else {
		imcal = aprefcal // ".fits"
		apcal = "ap" // aprefcal
		}

# Copy cal reference aperture file to local database/
	    apcalpath = adir // "database/" // apcal
	    apcalnew = "database/" // apcal
	    if (!access (apcalnew)) {
		print ("TEXTRACT: ",apcalpath," => ", apcalnew)
		copy (apcalpath, apcalnew)
		}
	    else
		print ("TEXTRACT: ",apcalnew," already exists")

# Copy cal reference image file to local directory
	    imcalpath = adir // imcal
	    if (!access (imcal)) {
		print ("TEXTRACT: Copying ",imcalpath, " to ",cwd)
		args = osfn (imcalpath) // " " // imcal
		lns (args)
		}
	    else
		print ("TEXTRACT: ",imcal," already exists")
	    }

#  Make temporary list of input files
	tchar = substr (ims,1,1)
	templist = mktemp ("tempexlist")
        if (tchar == '@') {
	    listfile = substr (ims,2,100)
	    copy (listfile, templist)
	    }
	else {
	    files (ims, >templist)
	    }
	list = templist
	while (fscan (list,imin) != EOF) {

# Make output file name
	    smatch (imin, ".ec")
	    if (smatch.found) {
		lroot = smatch.start - 1
		imroot = substr (imin, 1, lroot)
		}
	    else {
		smatch (imin, ".fits")
		if (smatch.found) {
		    lroot = smatch.start - 1
		    imroot = substr (imin, 1, lroot)
		    }
		else {
		    imroot = imin
		    }
		}
	    imin = imroot // ".fits"
	    outsky = imroot // "_sky.ec.fits"
	    outsci = imroot // "_sci.ec.fits"
	    outcal = imroot // "_cal.ec.fits"

# Make sure this is not already an extracted image
	    imgetp.default = "0"
	    imgetp (imin, "TEXTRACT")
	    if (imgetp.value != "0") {
		if (vb) {
		    print ("TEXTRACT: ", imin, " is already an extracted image")
		    print ("TEXTRACT = ", imgetp.value)
		    }
		next
		}
	    if (db) {
		if (aprefsci != "" && aprefsky != "")
		    print ("TEXTRACT: ", imin, " -> ", outsci," and ", outsky)
		else if (aprefsci != "" && aprefcal != "")
		    print ("TEXTRACT: ", imin, " -> ", outsci," and ", outcal)
		else if (aprefsci != "")
		    print ("TEXTRACT: ", imin, " -> ", outsci)
		else if (aprefcal != "")
		    print ("TEXTRACT: ", imin, " -> ", outcal)
		else if (aprefsky != "")
		    print ("TEXTRACT: ", imin, " -> ", outsky)
		}

# Set up apsum parameters
	    apsum.apertures = "1-52"
	    apsum.format = "echelle"
	    apsum.profiles = ""
	    apsum.interactive = no
	    apsum.find = no
	    apsum.recenter = no
	    apsum.resize = no
	    apsum.edit = no
	    apsum.trace = no
	    apsum.fittrace = no
	    apsum.extract = yes
	    apsum.extras = no
	    apsum.review = yes
	    apsum.line = INDEF
	    apsum.nsum = 10
	    apsum.background = "none"
	    apsum.weights = "none"
	    apsum.pfit = "fit1d"
	    apsum.clean = no
	    apsum.skybox = 1
	    apsum.saturation = INDEF
	    apsum.readnoise = "0."
	    apsum.gain = "1."
	    apsum.lsigma = 4.
	    apsum.usigma = 4.
	    apsum.nsubaps = 1

# Set time of operation
	    gettime ()
	    history = gettime.date // " " // gettime.time // " extracted from " // imin
	    hedit.show = vb

# Extract object spectrum to echelle file
	    if (aprefsci != "") {
		apsum.references = imsci
		print ("TEXTRACT: Extracting ",outsci)
		apsum (imin, output=outsci)
		hedit (outsci, "BANDID2", "echelle order", add+, verify-)
		hedit (outsci, "TEXTRACT", history, add+, verify-)
		hedit (outsci, "TEXAPREF", imscipath, add+, verify-)
		}

# Extract calibration spectrum to echelle file
	    if (aprefcal != "") {
		apsum.references = imcal
		print ("TEXTRACT: Extracting ",outcal)
		apsum (imin, output=outcal)
		hedit (outcal, "BANDID2", "echelle order", add+, verify-)
		hedit (outcal, "TEXTRACT", history, add+, verify-)
		hedit (outcal, "TEXAPREF", imcalpath, add+, verify-)
		}

# Extract background spectrum to echelle file
	    if (aprefsky != "") {
		apsum.references = imsky
		print ("TEXTRACT: Extracting ",outsky)
		apsum (imin, output=outsky)
		hedit (outsky, "BANDID2", "echelle order", add+, verify-)
		hedit (outsky, "TEXTRACT", history, add+, verify-)
		hedit (outsky, "TEXAPREF", imskypath, add+, verify-)
		}
	    }

	if (deltemp)
	    delete (templist)

end

# Sep 13 2007	New task based on hectospec.hextract
# Sep 18 2007	Allow extraction of any of the three apertures of data
