# QTRES - Process TRES image from a raw data file, removing cosmic rays, to a spectrum
# January 17, 2008
# By Doug Mink, SAO Telescope Data Center

procedure ctres (images)

string	images=""	{prompt="Images to process"}
string	imdate=""	{prompt="Date for images"}
string	instdir=""	{prompt="Directory containing date directory"}
bool	fixpreamp=yes	{prompt="Run tpreamp to fix time constant (yes or no)"}
bool	fixbias=no	{prompt="Fix bias section (yes or no)"}
real	gaincorr=1.0	{prompt="Gain correction for amp2 to amp1"}
bool	flatten=yes	{prompt="Run TFLAT to Flatten image (yes or no)"}
string  normfile=""     {prompt="Pixel to pixel tmakeflat normalization file"}
string	normdir="tresdata$"	{prompt="Directory for normalization file"}
bool	deltemp=yes	{prompt="Delete intermediate image files (y or n)"}
bool	saveold=yes	{prompt="Save unprocessed image files (y or n)"}
bool	verbose=yes	{prompt="Print process progress (y or n)"}

begin

file	imroot, imin, impath, templist, imlist
string	ims, imd, ind
bool	vb, dt, sv

	vb = verbose
	dt = deltemp
	sv = saveold
	imd = imdate
	ind = instdir
	ims = images

	if (ims == "") {
	    print ("CTRES: Reduce TRES images in a different directory")
	    print ("Usage: ctres images with both amps to be corrected")
	    return
	    }

#  Make temporary list of input files
	templist = mktemp ("tempctreslist")
	files (ims, >templist)

# Copy images to current directory
	if (ind != "") {
	    fixdir (ind)
	    impath = fixdir.newdir
	    list = templist
	    while (fscan (procfiles,imin) != EOF) {
		if (imd != "")
		    impath = impath // imd // "/" // imin
		else
		    impath = impath // imin
		if (!access (imin))
		    copy (impath, imin)
	    }
	}

# Remove overscan, trim, and merge amplifiers of images
# and remove pixel to pixel variation from images (including IR fringing)
	tproc.gaincorr = gaincorr
	tproc.fixbias = fixbias
	tproc.fixpreamp = fixpreamp
	tproc.normfile = normfile
	tproc.normdir = normdir
	tproc.deltemp = dt
	tproc.saveold = sv
	tproc.verbose = vb
	tproc (ims)

# Remove cosmic rays from list or list file of images
	tcosmic (ims)

# Extract spectra
	textract.aproot = ""
	textract.apdir = ""
	textract.deltemp = dt
	textract.verbose = vb
	textract (imin)

	if (dt)
	    delete (templist)

end

# Jan 17 2008	New task for quick look with cosmic ray removal
