# QTRES - Process a single TRES image from a raw data file to a spectrum
# January 17, 2008
# By Doug Mink, SAO Telescope Data Center

procedure qtres (image)

string	image=""	{prompt="Image to process"}
string	imdate=""	{prompt="Date for image"}
string	instdir=""	{prompt="Directory containing date directory"}
real	gaincorr=1.0	{prompt="Gain correction for amp2 to amp1"}
string  normfile=""     {prompt="Pixel to pixel tmakeflat normalization file"}
string  normdir="tresdata$"     {prompt="Directory for normalization file"}
bool	fixpreamp=yes	{prompt="Fix preamp time constant (yes or no)"}
bool	fixbias=no	{prompt="Fix bias section (yes or no)"}
bool	deltemp=yes	{prompt="Delete intermediate image files (y or n)"}
bool	saveold=yes	{prompt="Save unprocessed image files (y or n)"}
bool	verbose=yes	{prompt="Print process progress (y or n)"}

begin

file	imroot, amp1, amp2, temp2, imin, impath
string	imx, ims, imsec, history, imex
string	imd
bool	vb, dt, sv

	vb = verbose
	dt = deltemp
	sv = saveold
	imx = ".fits"
	imd = imdate
	imin = image

	if (imin == "") {
	    print ("QTRES: Reduce a single TRES image in a different directory")
	    print ("Usage: qtres image with both amps to be corrected")
	    return
	    }

# Copy image to current directory
	if (instdir != "") {
	    fixdir (instdir)
	    impath = fixdir.newdir
	    if (imd != "")
		impath = impath // imd // "/" // imin
	    else
		impath = impath // imin
	    if (!access (imin))
		copy (impath, imin)
	    }

# Remove overscan, trim, and merge amplifiers of image
# Then remove pixel to pixel variation from image (including IR fringing)
	tproc.gaincorr = gaincorr
	tproc.fixbias = fixbias
	tproc.fixpreamp = fixpreamp
	tproc.flatten = yes
	tproc.normfile = normfile
	tproc.normdir = normdir
	tproc.deltemp = dt
	tproc.saveold = sv
	tproc.verbose = vb
	tproc (imin)

# Extract spectra
	textract.aproot = ""
	textract.apdir = ""
	textract.deltemp = dt
	textract.verbose = vb
	textract (imin)

end

# Oct 12 2007	New task for quick look
# Dec 11 2007	Remove old declaration of instdir from before it was a parameter

# Jan 16 2008	Add fixpreamp parameter
# Jan 17 2008	Use tproc instead of tpreproc and tflatten
