# TPROC - Run image processing tasks on both TRES amps, combine, and flatten
# January 17, 2008
# By Doug Mink, SAO Telescope Data Center

procedure tproc (images)

string	images=""	{prompt="Images to merge"}
real	gaincorr=1.0	{prompt="Gain correction for amp2 to amp1"}
bool	fixpreamp=yes	{prompt="Make preamp correction (yes or no)"}
bool	fixbias=no	{prompt="Fix bias section (yes or no)"}
string  normfile=""     {prompt="Pixel to pixel tmakeflat normalization file"}
string  normdir="tresdata$"     {prompt="Directory for normalization file"}
bool	flatten=yes	{prompt="Run tflat on each image (yes or no)"}
bool	deltemp=yes	{prompt="Delete intermediate image files (yes or no)"}
bool	saveold=yes	{prompt="Save unprocessed image files (yes or no)"}
bool	verbose=yes	{prompt="Print process progress (yes or no)"}

struct *procfiles
begin

file	imroot, amp1, amp2, temp1, temp2, temp3, im2, imin, templist
file	imfile, newfile, imlist
string	chip1, chip2, arg1, imx, ims, imsec, history, tchar, imex
string	binning, nd, nf
bool	vb, dt, sv, fp, fb, flat
real	gc

	vb = verbose
	dt = deltemp
	sv = saveold
	fp = fixpreamp
	fb = fixbias
	flat = flatten
	gc = gaincorr
	nf = normfile
	nd = normdir
	imx = ".fits"
	ims = image

	if (ims == "") {
	    print ("TPROC: Process TRES image files to extraction")
	    print ("Usage: tproc im1,im2,...,imn")
	    print ("Usage: tproc *.fits")
	    print ("Usage: tproc @image_list_file")
	    return
	    }

	if (!defpac ("mscred"))
	    mscred

#  Make temporary list of input files
	tchar = substr (ims,1,1)
	templist = mktemp ("temproclist")
	if (tchar == '@') {
	    imlist = substr (ims,2,100)
	    copy (imlist, templist)
	    }
	else {
	    files (ims, >templist)
	    }

# Read through input files
	procfiles = templist
        while (fscan (procfiles,imin) != EOF) {

# Process files as images
	    tpreproc (imin,gaincorr=gc,fixpreamp=fp,fixbias=fb,deltemp=dt,saveold=sv,verbose=vb)

# Flatfield correct spectra
	    if (flat)
		tflat (imin,normfile=nf,normdir=nd,deltemp=dt,saveold=sv,verbose=vb)
	    }

	if (dt)
	    delete (templist)
	procfiles = ""
end

# Jan 09 2008	New program to process list of TRES images
# Jan 17 2008	Update normfile and normdir prompts
